/*
	BrwMon1.prg  -  2009.01.23
	Free, open source code to use with Harbour/miniGUI
	Sorry for my "no good" english, I'm learnig it too.

	What if your program automatically shows secondary database info depending on
	record selected by end user at the primary database, but him (the user) click
	with the right button... this not jump to your ON CHANGE event code.
	Left click jumps correctly the ON CHANGE event
	Right click appears to no change the RECNO() pointer, even with SET BROWSESYNC ON

	This is the set of functions (you can be an alfa tester)

		BRWMonitorBegin()
		BRWMonitorTimerPause()
		BRWMonitorTimerResume()

		BRWMonitorTimerRelease()
		BRWMonitor()

	The code is tiny, but is fully commented.

	-> This process uses a timer wich periodically compares de VALUE property
		of the browse object with its database RECNO() and jumps to a function
		you can write in your own code i order to handle it.

	-----------------------------------------------------------------------------
	Personal note:
		Maybe there is a command... and I don't know how to handle this behavior.
		(I downloaded miniGUI 2.6.7 few days ago in order to learn windows programing)
		If this is a stupid thing, please let me know (also your changes)
		arturo.garcia.vazquez@yahoo.com.mx
*/

//==============================================================================
// you can include this routines in your main code with   #include "BrwMon1.prg"
//==============================================================================
#define cBRWMonitorTimerName "BRWMonitorTimerName"


/*------------------------------------------------------------------------------
	sintax: BRWMonitor( [ 0 ] )  --> returns previous value of a control public var
	If for debug purposes you want the BRWMonitor set of functions be ignored,
	that is, returnig inmediately with no executing its process at all, you can
	use  BRWMonitorTimerPause()  		// if timer is already running
	and  BRWMonitor( 0 ) 	// to nullify the action of BRWMonitor set of functions
*/
Function BRWMonitor( pnCodeToSet )
LOCAL nCodeToReturn

	IF .NOT. TYPE("nBRWPublicCode") == "N"
		PUBLIC nBRWPublicCode := 1 		// default first status code is:  1 (ON)
	ENDIF
	
	nCodeToReturn := nBRWPublicCode		// allways returns previous status

	IF PCOUNT() = 1
		nBRWPublicCode := pnCodeToSet		// optional, new code to set
	ENDIF
	
Return nBRWPublicCode


/*------------------------------------------------------------------------------
	sintax: BRWMonitorBegin( <cWindowNameToUse>, <arrayBrowseObjects>, ;
										<cUserFunction>, <nIntervalMilliseconds> )  -> Nil
	Defines a timer in <cWindowNameToUse> and starts the monitoring of databases
	<arrayBrowseObjects>
			is an array with the specifics of each browse object to be monitorizing
			in the following format:
				{
					{ <cWindowName1> , <cBrowseName1> , <cDatabaseAliasName1> }  ,  ;
					{ <cWindowName2> , <cBrowseName2> , <cDatabaseAliasName2> }  ,  ;
					...
					{ <cWindowNameN> , <cBrowseNameN> , <cDatabaseAliasNameN> }     ;
				}
	<cUserFunction> 
			is an user defined function specified surrounded with quotes
			and without parenthesis wich be called internally (with 2 parameters, see
			below) each time the monitor detects a diference between
			<cWindowName>.<cBrowseName>.VALUE   and   its respective   RECNO()
			caused when an end user selects a record in the main database with
			the right button of the mouse (which not shoot the ON CHANGE event).
			Inside this user defined function you are able to update the visuals
			of a secondary database.
			Your user defined function *must* receive 2 parameters of type array
			<alBrowseObjectList>  and  <arrayBrowseObjects>
			the first array is a list of logicals .T./.F. indicating with .T.
			If a browse pointer (VALUE property) has been changed and differs of its
			respective RECNO()
			The second array parameter received for your user defined function is
			the same array you provided with your Browse Objects List
			IMPORTANT.
					To avoid problems with recursive calls, BRWMonitorTimerPause()
					is executed just before your user defined function is called.
					Your user defined function *must* return a .T. or .F.,
					but only when .T. is returned a call to BRWMonitorTimerResume()
					is made.
	<nIntervalMilliseconds>
			Is the period of time to use as INTERVAL in the timer that monitorizes
			your databases. Maybe 1/4 or 1/2 second seems to be Ok, but this is on you.
*/
Function BRWMonitorBegin( pcWindowNameToUse, pacBrwObjectsToMonitor, pcProcedureName, pnInterval )

	IF BRWMonitor() = 0
		Return Nil
	ENDIF
	
	DEFINE TIMER &cBRWMonitorTimerName OF &pcWindowNameToUse ;
		INTERVAL pnInterval ;
		ACTION BRWMonitorTimerTask( pcWindowNameToUse, pacBrwObjectsToMonitor, pcProcedureName )

Return Nil


/*------------------------------------------------------------------------------
	sintax: BRWMonitorTimerPause( <cWindowName> )  -> Nil
	Pauses the monitorizing process. (disables the timer)
	<cWindowName>
			is the same  cWindowNameToUse  you specified in BRWMonitorBegin()
*/
Function BRWMonitorTimerPause( pcWindowNameToUse )
	IF BRWMonitor() = 0
		Return Nil
	ENDIF
	
	setproperty( pcWindowNameToUse, cBRWMonitorTimerName, "ENABLED", .F. )
Return Nil


/*------------------------------------------------------------------------------
	sintax: BRWMonitorTimerResume( <cWindowName> )  -> Nil
	Resumes the monitorizing process. (reenables the timer)
	<cWindowName>
			is the same  cWindowNameToUse  you specified in BRWMonitorBegin()
*/
Function BRWMonitorTimerResume( pcWindowNameToUse )
	IF BRWMonitor() = 0
		Return Nil
	ENDIF
	
	setproperty( pcWindowNameToUse, cBRWMonitorTimerName, "ENABLED", .T. )
Return Nil


/*------------------------------------------------------------------------------
	sintax: BRWMonitorTimerRelease( <cWindowName> )  -> Nil
	Stops and destroys the monitorizing process. (release the timer)
	<cWindowName>
			is the same  cWindowNameToUse  you specified in BRWMonitorBegin()
*/
Function BRWMonitorTimerRelease( pcWindowNameToUse )
	IF BRWMonitor() = 0
		Return Nil
	ENDIF
	
	setproperty( pcWindowNameToUse, cBRWMonitorTimerName, "RELEASE", .T. )
Return Nil


/*------------------------------------------------------------------------------
	internal monitorizing task, programer not need to call this function
*/
Function BRWMonitorTimerTask( pcWindowNameToUse, pacBrwObjectsToMonitor, pcProcedureName )
LOCAL n, ;
		nCurrentValueFromBrwObject, ;
		nCurrentValueFromRecno, ;
		cRecnoExpression, ;
		lUDFNeedToBeCalled, ;
		lBrwWantToConinue, ;
		cCallUDFwithStatusList
PRIVATE alBRWList := {}		// requires PRIVATE to use with & macro
PRIVATE acBrwObjectsToMonitor := pacBrwObjectsToMonitor

	IF BRWMonitor() = 0
		Return Nil
	ENDIF
	
	BRWMonitorTimerPause( pcWindowNameToUse )	// avoid recursive timer calls
	
	// let's compare "Form_name"."Browse_name".VALUE  =  "dbfAliasName"->( RECNO() )
	// and stores each result in alBRWList array
	lUDFNeedToBeCalled := .F.
	FOR n := 1 TO len(pacBrwObjectsToMonitor)
		nCurrentValueFromBrwObject := getproperty( pacBrwObjectsToMonitor[n,1], pacBrwObjectsToMonitor[n,2], "VALUE" )
		cRecnoExpression := pacBrwObjectsToMonitor[n,3] + "->( RECNO() )"
		nCurrentValueFromRecno := &cRecnoExpression
		aAdd( alBRWList, ( nCurrentValueFromBrwObject != nCurrentValueFromRecno ) )
		IF alBRWList[n] = .T.
			lUDFNeedToBeCalled := .T.
		ENDIF
	NEXT n

	lBrwWantToConinue := .T.
	IF lUDFNeedToBeCalled	// let us call user defined function
		// User defined function *MUST* return .T./.F. if want (or not) resume monitoring
		cCallUDFwithStatusList := pcProcedureName + "( alBRWList, acBrwObjectsToMonitor )"
		lBrwWantToConinue := &cCallUDFwithStatusList
	ENDIF
	
	IF lBrwWantToConinue
		BRWMonitorTimerResume( pcWindowNameToUse )	// resume timer "monitoring"
	ENDIF
	
Return Nil

